// SPDX-License-Identifier: GPL-2.0-only
#ifndef __LINUX_DMA_QCOM_ADM_H
#define __LINUX_DMA_QCOM_ADM_H

#include <linux/types.h>

struct qcom_adm_peripheral_config {
	u32 crci;
	u32 mux;
};

#endif /* __LINUX_DMA_QCOM_ADM_H */
