/* SPDX-License-Identifier: (GPL-2.0 OR MIT) */
#ifndef __DT_BINDINGS_POWER_RK3576_POWER_H__
#define __DT_BINDINGS_POWER_RK3576_POWER_H__

/* VD_NPU */
#define RK3576_PD_NPU		0
#define RK3576_PD_NPUTOP	1
#define RK3576_PD_NPU0		2
#define RK3576_PD_NPU1		3

/* VD_GPU */
#define RK3576_PD_GPU		4

/* VD_LOGIC */
#define RK3576_PD_NVM		5
#define RK3576_PD_SDGMAC	6
#define RK3576_PD_USB		7
#define RK3576_PD_PHP		8
#define RK3576_PD_SUBPHP	9
#define RK3576_PD_AUDIO		10
#define RK3576_PD_VEPU0		11
#define RK3576_PD_VEPU1		12
#define RK3576_PD_VPU		13
#define RK3576_PD_VDEC		14
#define RK3576_PD_VI		15
#define RK3576_PD_VO0		16
#define RK3576_PD_VO1		17
#define RK3576_PD_VOP		18

#endif
