/* SPDX-License-Identifier: GPL-2.0-only */
/*
 *
 *  Copyright (C) 2016 Martin Blumenstingl <martin.blumenstingl@googlemail.com>
 *  Copyright (C) 2017 Hauke Mehrtens <hauke@hauke-m.de>
 */
#ifndef _DT_BINDINGS_MIPS_LANTIQ_RCU_GPHY_H
#define _DT_BINDINGS_MIPS_LANTIQ_RCU_GPHY_H

#define GPHY_MODE_GE	1
#define GPHY_MODE_FE	2

#endif /* _DT_BINDINGS_MIPS_LANTIQ_RCU_GPHY_H */
