/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2020 Nicolas Saenz Julienne
 * Author: Nicolas Saenz Julienne <nsaenzjulienne@suse.com>
 */

#ifndef _DT_BINDINGS_RASPBERRYPI_FIRMWARE_RESET_H
#define _DT_BINDINGS_RASPBERRYPI_FIRMWARE_RESET_H

#define RASPBERRYPI_FIRMWARE_RESET_ID_USB	0
#define RASPBERRYPI_FIRMWARE_RESET_NUM_IDS	1

#endif
